print("import: 'pyclustering'")
import pyclustering

print("import: 'pyclustering.cluster'")
import pyclustering.cluster

print("import: 'pyclustering.cluster.examples'")
import pyclustering.cluster.examples

print("import: 'pyclustering.cluster.tests'")
import pyclustering.cluster.tests

print("import: 'pyclustering.cluster.tests.integration'")
import pyclustering.cluster.tests.integration

print("import: 'pyclustering.cluster.tests.unit'")
import pyclustering.cluster.tests.unit

print("import: 'pyclustering.container'")
import pyclustering.container

print("import: 'pyclustering.container.examples'")
import pyclustering.container.examples

print("import: 'pyclustering.container.tests'")
import pyclustering.container.tests

print("import: 'pyclustering.container.tests.unit'")
import pyclustering.container.tests.unit

print("import: 'pyclustering.core'")
import pyclustering.core

print("import: 'pyclustering.core.tests'")
import pyclustering.core.tests

print("import: 'pyclustering.gcolor'")
import pyclustering.gcolor

print("import: 'pyclustering.gcolor.examples'")
import pyclustering.gcolor.examples

print("import: 'pyclustering.gcolor.tests'")
import pyclustering.gcolor.tests

print("import: 'pyclustering.nnet'")
import pyclustering.nnet

print("import: 'pyclustering.nnet.examples'")
import pyclustering.nnet.examples

print("import: 'pyclustering.nnet.tests'")
import pyclustering.nnet.tests

print("import: 'pyclustering.nnet.tests.integration'")
import pyclustering.nnet.tests.integration

print("import: 'pyclustering.nnet.tests.unit'")
import pyclustering.nnet.tests.unit

print("import: 'pyclustering.samples'")
import pyclustering.samples

print("import: 'pyclustering.tests'")
import pyclustering.tests

print("import: 'pyclustering.utils'")
import pyclustering.utils

print("import: 'pyclustering.utils.examples'")
import pyclustering.utils.examples

print("import: 'pyclustering.utils.tests'")
import pyclustering.utils.tests

print("import: 'pyclustering.utils.tests.integration'")
import pyclustering.utils.tests.integration

print("import: 'pyclustering.utils.tests.unit'")
import pyclustering.utils.tests.unit

